<?php
/*
* place for functions used by different interface implementations
*
*/

/*
* calculates seconds to download $bytes at $speed kB/s
*
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
function eta($bytes, $speed)
{
	if($speed == "paused") { return "&infin;"; }
	if($speed == "finished") { return "0"; }
	if($speed <= 0) { return "&infin;"; }
	
	$seconds = $bytes / (1024*$speed);
	if($seconds > 365*24*60*60) { return "&infin;"; }
	return $seconds;
}

/*
* calculates percentage of available chunks by a chunk array
* 0 => not available
* 1 =>  available
*
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
function availability($chunks)
{
	$unavail = 0;
	$avail = 0;
	$len = strlen($chunks);
	for ($i = 0; $i < $len; $i++) {
		if($chunks[$i] == 0) { $unavail++; }
		if($chunks[$i] == 1) { $avail++; }
		
	}
	
	if(($avail + $unavail) == 0) {
		return 100;
	}
	return ($avail * 100)/($avail + $unavail);
}


/*
* byteSize() formats string $size to a byte size value
*
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
function byteSize($size)
{
	$pos = strcspn($size, "bkmgKMG");
	switch(substr($size,$pos)) {                    //the smallest size of $size should be the "0b"
		case "kb":
		case "k": return $size*1024;
		case "mb":
		case "M": return $size*1024*1024;
		case "gb":
		case "G": return $size*1024*1024*1024;
		default: return ($size*1);          //this case should match Byte "123b" or "0b" at last
	}
}

/*
* merge_objects() merges  all objects  in $object_array 
* $vars_array is a white list ($choice = TRUE) or a black list (FALSE)
*
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
function merge_objects($object_array, $vars_array, $choice) {

	$object_merged = $object_array[0]; //first element is a container
	array_shift($object_array);
	
	foreach($object_array as $object) {
	
		$vars = get_object_vars($object);
		foreach($vars as $var => $value)
		{
			if($choice AND !in_array($var, $vars_array)) { //$vars_array is white list
				continue;
			}
			if(!$choice AND in_array($var, $vars_array)) { //$vars_array is black list
				continue;
			}
			
			if(empty($object_merged->{$var})) {
				continue;
			} elseif(is_string($object_merged->{$var})) {
				$object_merged->{$var} .= $value;
			} elseif(is_int($object_merged->{$var})) {
				$object_merged->{$var} += $value;
			} elseif(is_array($object_merged->{$var})) {
				$object_merged->{$var} = array_merge($object_merged->{$var}, $value);
			}
		}
	}
	return $object_merged;
}

/*
* unique_objects uniques an array of objects by the variable name $unique
* if $collect = TRUE, equal objects (in variable $unique) are stored in a subarray
*
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
function unique_objects($object_array, $unique, $collect) {

	$unique_array = array(); // key => object
	
	foreach($object_array as $object) {

		$value = serialize($object->{$unique}); //serialze is used to handle different types  of variables in objects, like arrays
		
		if(!empty($value)) {  //the object have a variable named $unique
			if(!array_key_exists($value, $unique_array)) {  //there were  no objects before with the same value called $unique
				$unique_array[$value] = $object;
			} elseif($collect) {
				if(!is_array($unique_array[$value])) { //create subarray
					$unique_array[$value] = array($unique_array[$value]);
					$unique_array[$value][] = $object;
				} else { //add to subarray
					$unique_array[$value][] = $object;
				}
			} else {
				$unique_array[$value] = $object;
			}
		} else {
			$unique_array[$value] = $object;
		}
	}
	
	return array_values($unique_array);
}

/*
* class Cache provide a felxible cache system
*
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
class Cache {
	
	/*
	* array of function names to set the caches
	*/
	var $cache_source_func;
	
	/*
	* create a new cache variabe and set the source function
	*/
	function cache_add($cache, $source_func) {
		if(!in_array($source_func, get_class_methods(get_class($this)))) {
			return FALSE;
		}
		$this->$cache = array(); //init cache variable
		$this->cache_source_func[$cache] = $source_func; //register source function
	}
	
	/*
	* get values from cache by keys
	*/
	function cache_get($cache, $keys, $keep_missing = FALSE) {
		
		$result = array();
		foreach((array) $keys as $key) {
			
			if(!is_null($value = $this->{$cache}[$key])) { //if key is set, collect value
				$result[$key] = $value;
			} elseif(!is_null($value = $this->cache_call_source($cache, $key))) { //get value by source and collect
				$result[$key] = $value;
			} elseif($keep_missing) { //if value could not get, set to NULL or leave it
				$result[$key] = NULL;
			} else {
				continue;
			}
		}
		
		//format return value
		if(is_array($keys)) {
			return $result;
		} else {
			return $result[$keys];
		}
	}
	
	/*
	* calls the source funtion to update the cache
	*/
	function cache_call_source($cache, $key) {
		
		//call source function
		$this->{$this->cache_source_func[$cache]}($key);
		
		if($this->{$cache}[$key] != NULL) {
			return $this->{$cache}[$key];
		} else {
			echo "error in cache_call_source: could not set cache: ".$cache."[".$key."]".BN;
			return NULL;
		}
	}
	
	/*
	* reset several caches or keys in a cache
	*/
	function cache_reset($caches = NULL, $keys = NULL) {
		
		//select all caches
		if($caches == NULL) {
			$caches = array_keys($this->cache_source_func);
		}
	
		foreach((array) $caches as $cache) {
			if($keys == NULL) {
				$this->$cache = array(); //clear all keys
			} else {
				foreach((array) $keys as $key) {
					unset($this->{$cache}[$key]); //clear selected keys
				}
			}
		}
		return TRUE;
	}
	
	/*
	* set a cache value
	*/
	function cache_set($cache, $key, $value) {
		if($value != NULL) {
			$this->{$cache}[$key] = $value;
			return TRUE;
		}
		echo "error in cache_set: value is NULL".BN;
		return FALSE;
	}
}
?>